
template <class T>
class ArrList {

    T* arr;
    int len;
    int head;
    int tail;

    // reallocate arr to ensure it can store at least n elements.
    void ensure_capacity(int n);

public:

    ArrList();

    // copy constructor (make a copy of an existing list)
    // ArrList a = b;   // a does not "exist" before assignment.
    ArrList(ArrList<T>& c);

    ~ArrList();

    // assignment operator; 
    //  ArrList a;   // a exists now.
    //  a = b;       // assignment operator
    ArrList<T>& operator=(ArrList<T>& c);

    // check if list is empty (head is same as tail).
    bool isEmpty();

    // returns number of elements in the list.
    int size();

    // get i-th element starting from tail (index 0).
    T get(int i);

    // set n at ith location.
    void set(int i,T n);

    // push (add) an integer onto the list (at head).
    void push(T n);

    // reverse of push.
    T pop();

    // remove element from tail (similar to pop, except from other end).
    T shift();

    // insert at the tail of the list (similar to push).
    void unshift(T n);

    // splice (index, length, replacewith).
    ArrList<T>& splice(int n);
    
    // at location n, remove l elements.
    ArrList<T>& splice(int n,int l);

    // at location n, remove l elements, replace with lst.
    ArrList<T>& splice(int n,int l,ArrList<T>& lst);
};

    
    // reallocate arr to ensure it can store at least n elements.
    
    template <class T>
    void ArrList<T>::ensure_capacity(int n){
        // e.g. double the list capacity, if needed.
        if(len > n)     // check if we need to do this...
            return;             // no, there's enough space there.
        int newlen = len;
        while(newlen <= n)    // until we can store n elements.
            newlen = newlen * 2;    // keep on doubling list length
        // allocate a new array of size newlen.
        T* newarr = new T[newlen];
        for(int i=0;i<size();i++)
            newarr[i] = get(i);       // copy old list into new list.
        delete[] arr;       // free old list.
        arr = newarr;       // setup new list.
        head = size();      // head is the end of the list.
        tail = 0;           // new list starts at 0.
        len = newlen;       // setup new length.
    }

    template <class T>
    ArrList<T>::ArrList(){
        len = 8;
        arr = new T[len];
        head = tail = 0;
    }

    // copy constructor (make a copy of an existing list)
    // ArrList a = b;   // a does not "exist" before assignment.
    template <class T>
    ArrList<T>::ArrList(ArrList<T>& c){
        len = c.len;
        arr = new T[len];
        head = tail = 0;
        for(int i=0;i<c.size();i++)
            push(c.get(i));
    }

    template <class T>
    ArrList<T>::~ArrList(){
        // clean up whatever we allocated in constructor
        delete[] arr;
        len = head = tail = 0;  // zero things out.
        arr = (T*)0;
    }

    // assignment operator; 
    //  ArrList a;   // a exists now.
    //  a = b;       // assignment operator
    template <class T>
    ArrList<T>& ArrList<T>::operator=(ArrList& c){    
        delete[] arr;       // free list that's already there.
        len = c.len;
        arr = new T[len];
        head = tail = 0;
        for(int i=0;i<c.size();i++)
            push(c.get(i));
    }

    // check if list is empty (head is same as tail).
    template <class T>
    bool ArrList<T>::isEmpty(){
        return head == tail;
    }

    // returns number of elements in the list.
    template <class T>
    int ArrList<T>::size(){
        return (head - tail + len) % len;
    }

    // get i-th element starting from tail (index 0).
    template <class T>
    T ArrList<T>::get(int i){
        if(i < 0)   // if negative, return starting from end.
            return arr[ (head + len + i) % len ];
        return arr[ (tail + i) % len ];
    }

    // set n at ith location.
    template <class T>
    void ArrList<T>::set(int i,T n){
        if(i < 0){
            arr[ (head + len + i) % len ] = n;
        }else{
            arr[ (tail + i) % len ] = n;
        }
    }

    // push (add) an integer onto the list (at head).
    template <class T>
    void ArrList<T>::push(T n){
        ensure_capacity(size() + 1);    // ensure we have enough space.
        arr[head] = n;      // insert at head.
        head = (head + 1) % len;        // move head to next slot.
    }

    // reverse of push.
    template <class T>
    T ArrList<T>::pop(){
        if(size() <= 0)     // this is a *really* bad error. 
            return arr[0];  // we want to exit program if this happens.
        head = (head + len - 1) % len;  // slot before head index.
        return arr[head];           // return element under head.
    }

    // remove element from tail (similar to pop, except from other end).
    template <class T>
    T ArrList<T>::shift(){
        T n = arr[tail];
        tail = (tail + 1) % len;        // move head to next slot.
        return n;
    }

    // insert at the tail of the list (similar to push).
    template <class T>
    void ArrList<T>::unshift(T n){
        ensure_capacity(size() + 1);    // ensure we have enough space.
        tail = (tail + len - 1) % len;        // move tail 1 back.
        arr[tail] = n;      // insert at tail.
    }

    // splice (index, length, replacewith).
    template <class T>
    ArrList<T>& ArrList<T>::splice(int n){
        // remove all elements after index n.
        while(size() > n)
            pop();
        return *this;
    }
    
    // at location n, remove l elements.
    template <class T>
    ArrList<T>& ArrList<T>::splice(int n,int l){
        ArrList<T> tlst;
        // remove all elements after index n.
        while(size() > n)
            tlst.push(pop());   // save everything after n.
        while(l-- > 0)
            tlst.pop();     // remove l saved elements.
        // put back the remainer.
        while(tlst.size()>0)
            push(tlst.pop());
        return *this;
    }

    // at location n, remove l elements, replace with lst.
    template <class T>
    ArrList<T>& ArrList<T>::splice(int n,int l,ArrList<T>& lst){
        //printf("before splice: len=%d, siz=%d, head=%d, tail=%d\n",len,size(),head,tail);
        ArrList<T> tlst;
        // remove all elements after index n.
        while(size() > n)
            tlst.push(pop());   // save everything after n.
        while(l-- > 0)
            tlst.pop();     // remove l saved elements.
        for(int i=0;i<lst.size();i++)   // replacement list.
            push(lst.get(i));
        // put back the remainer.
        while(tlst.size()>0)
            push(tlst.pop());
            // printf("after splice: len=%d, siz=%d, head=%d, tail=%d\n",len,size(),head,tail);            
        return *this;
    }

