#include <SDL/SDL.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <cstdio>
#include <cstdlib>
#include <cstdarg>
#include <ctime>
#include <cmath>
#include <cstring>

class Circle {
public:
    double x,y;
    double rad;
    double r,g,b;
    double speed,direction;

    Circle() {
                x = (rand()/(float)RAND_MAX) * 2 - 1; //x
                y = (rand()/(float)RAND_MAX) * 2 - 1; // y
                rad = (rand()/(float)RAND_MAX) * 0.02; // rad
                r  =rand()/(float)RAND_MAX; //r
                g = rand()/(float)RAND_MAX; //g
                b = rand()/(float)RAND_MAX; //b
                speed = (rand()/(float)RAND_MAX) * 0.02; //speed
                direction = (rand()/(float)RAND_MAX) * (2.0 * atan(1)*4.0); //
    }

    Circle(double x,double y,double rad,double r,double g,double b, double speed,double direction){
        this->x = x;
        this->y = y;
        this->rad = rad;
        this->r = r;
        this->g = g;
        this->b = b;
        this->speed = speed;
        this->direction = direction;
    }

    void draw(){
        double x = this->x;
        double y = this->y;
        double pi = atan(1)*4.0;     // eg. 3.14...
        for(double theta = 0; theta < 2*pi; theta += pi / 10){
            double y1 = y + sin(theta) * rad;
            double x1 = x + cos(theta) * rad;
            double y2 = y + sin(theta + pi/10) * rad;
            double x2 = x + cos(theta + pi/10) * rad;
            // key thing of our code.
            glBegin( GL_TRIANGLES );
                glColor3d( r, g, b );
                glVertex2d( x,y );
                glColor3d( r, g, b );
                glVertex2d( x1,y1 );
                glColor3d( r, g, b );
                glVertex2d( x2,y2 );
            glEnd( );
        }
    }

    void move(){
        double s = sin(direction);
        double c = cos(direction);
        x = x + c * speed;
        y = y + s * speed;
        if(x < -1)
            x += 2;
        if(y < -1)
            y += 2;
        if(x > 1)
            x -= 2;
        if(y > 1)
            y -= 2;
    }

};

void swapd(double &a,double &b){
    double t = a;
    a = b;
    b = t;
}


// main to get the whole thing going.
int main( int argc, char** argv ) {

    srand(time(NULL));

    // First, initialize SDL's video subsystem. 
    if( SDL_Init( SDL_INIT_VIDEO ) < 0 ) {
        printf("failed to init video: %s",SDL_GetError());
        SDL_Quit( );
        return -1;
    }

    // Information about the current video settings. 
    const SDL_VideoInfo* info = SDL_GetVideoInfo( );
    if(!info){
        printf("failed to query video: %s",SDL_GetError());
        SDL_Quit( );
        return -1;
    }

    // setup opengl params
    //SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
    //SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
    //SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
    //SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
    //SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
    
    int flags = SDL_OPENGL; // | SDL_FULLSCREEN;
    int width = 640;
    int height = 480;
    int bpp = info->vfmt->BitsPerPixel;

    // Set the video mode
    if( SDL_SetVideoMode( width, height, bpp, flags ) == 0 ) {
        printf("failed init video mode: %s",SDL_GetError());
        SDL_Quit( );
        return -1;
    }

    //float ratio = (float) width / (float) height;

    glShadeModel( GL_SMOOTH );
    //glCullFace( GL_BACK );  // remove triangles not facing us.
    //glFrontFace( GL_CCW );  // how we define 'front"
    //glEnable( GL_CULL_FACE );
    glClearColor( 0, 0, 0, 0 ); // black
    glViewport( 0, 0, width, height );  // our "screen"
    //glMatrixMode( GL_PROJECTION );  // set projection 
    //glLoadIdentity( );      // load identity matrix into projection matrix.
    //gluPerspective( 60.0, ratio, 1, 1024.0 ); // setup perspective matrix.

    Circle clst[500];
    int clst_siz = 500;

    for(;;){

        // read key strokes
        SDL_Event event;
        while( SDL_PollEvent( &event ) ) {
            switch( event.type ) {
            case SDL_KEYDOWN:
                printf("The %s(%d) key was pressed!\n",
                    SDL_GetKeyName(event.key.keysym.sym),event.key.keysym.sym);
                if(event.key.keysym.sym == SDLK_ESCAPE){
                    SDL_Quit( );
                    return 0;
                }
            break;
            case SDL_KEYUP:
                printf("The %s(%d) key was released!\n",
                    SDL_GetKeyName(event.key.keysym.sym),event.key.keysym.sym);
                break;            
           case SDL_MOUSEMOTION:
                printf("Mouse moved by %d,%d to (%d,%d)\n", 
                    event.motion.xrel, event.motion.yrel,
                    event.motion.x, event.motion.y);
                break;
            case SDL_MOUSEBUTTONDOWN:
                printf("Mouse button %d pressed at (%d,%d)\n",
                    event.button.button, event.button.x, event.button.y);
                break;
            case SDL_QUIT:      // window closed.
                SDL_Quit( );
                return 0; 
                break;
            }
        }

        


        // draw stuff to screen...
        //glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
        glClear( GL_COLOR_BUFFER_BIT );        
        //glMatrixMode( GL_MODELVIEW );
        //glLoadIdentity( );
        //glTranslatef( 0.0, 0.0, -5.0 );
        //glRotatef( angle, 0.0, 1.0, 0.0 );

        // key thing of our code.
        //glBegin( GL_TRIANGLES );
        //    glColor3f( 255, 0, 0 );
        //    glVertex2f( -0.5,-0.5 );
        //    glColor3f( 0, 255, 0 );
        //    glVertex2f( 0.5,-0.5 );
        //    glColor3f( 0, 0, 255 );
        //    glVertex2f( 0.5,0.5 );
        //glEnd( );
        
        for(int i=0;i<clst_siz;i++){
            clst[i].draw();
            clst[i].move();
        }

        for(int i=0;i<clst_siz-1;i++){
            for(int j=i+1;j<clst_siz;j++){
                double d = sqrt((clst[i].x - clst[j].x)*(clst[i].x - clst[j].x) + (clst[i].y - clst[j].y)*(clst[i].y - clst[j].y));
                if(d < (clst[i].rad + clst[j].rad)){    // handle collision
                    swapd(clst[i].speed,clst[j].speed);
                    swapd(clst[i].direction,clst[j].direction);
                }else if(d < (clst[i].rad + clst[j].rad)*3.0 ){
                    double ad = (clst[i].direction+clst[j].direction) / 2;
                    double as = (clst[i].speed+clst[j].speed) / 2;
                    clst[i].speed += (as - clst[i].speed) * 0.1;
                    clst[i].direction += (ad - clst[i].direction) * 0.1;
                    if(rand()/(double)RAND_MAX < 0.01){
                        clst[i].direction -= atan(1)*4;
                    }
                    //clst[j].speed += (as - clst[j].speed) * 0.1;
                    //clst[j].direction += (ad - clst[j].direction) * 0.1;
                }
            }
        }

        SDL_GL_SwapBuffers( );
        SDL_Delay(30);
    }
    
    SDL_Quit( );
    return 0;
}



